@@ -63,10 +63,17 @@ ncmpi_def_var(int         ncid,    /* IN:  file ID */
         goto err_check;
     }
 
-    /* For CDF-1 and CDF-2 files, only classic types are allowed. */
-    if (pncp->format < NC_FORMAT_CDF5 && type > NC_DOUBLE) {
-        DEBUG_ASSIGN_ERROR(err, NC_ESTRICTCDF2)
-        goto err_check;
+    /* For NC_FORMAT_CLASSIC, NC_FORMAT_64BIT_OFFSET, and
+     * NC_FORMAT_NETCDF4_CLASSIC files, only classic types are allowed. */
+    if (type > NC_DOUBLE) {
+        if (pncp->format <= NC_FORMAT_64BIT_OFFSET) {
+            DEBUG_ASSIGN_ERROR(err, NC_ESTRICTCDF2)
+            goto err_check;
+        }
+        else if (pncp->format == NC_FORMAT_NETCDF4_CLASSIC) {
+            DEBUG_ASSIGN_ERROR(err, NC_ESTRICTNC3)
+            goto err_check;
+        }
     }
 
     /* Argument ndims is of type "int". Its max value will be less than
@@ -209,8 +216,10 @@ err_check:
     pncp->vars[*varidp].recdim = -1;   /* if fixed-size variable */
     pncp->vars[*varidp].shape  = NULL;
     if (ndims > 0) {
-        if (dimids[0] == pncp->unlimdimid) /* record variable */
+        if (dimids[0] == pncp->unlimdimid) { /* record variable */
             pncp->vars[*varidp].recdim = pncp->unlimdimid;
+            pncp->nrec_vars++;
+        }
 
         pncp->vars[*varidp].shape = (MPI_Offset*)
                                     NCI_Malloc(ndims * SIZEOF_MPI_OFFSET);
@@ -379,9 +388,14 @@ ncmpi_inq_vartype(int      ncid,    /* IN:  file ID */
     /* check whether variable ID is valid */
     if (varid < 0 || varid >= pncp->nvars) DEBUG_RETURN_ERROR(NC_ENOTVAR)
 
+    *xtypep = pncp->vars[varid].xtype;
+    return NC_NOERR;
+
+#if 0
     /* calling the subroutine that implements ncmpi_inq_vartype() */
     return pncp->driver->inq_var(pncp->ncp, varid, NULL, xtypep, NULL,
                                  NULL, NULL, NULL, NULL, NULL);
+#endif
 }
 
 /*----< ncmpi_inq_varndims() >-----------------------------------------------*/
@@ -406,9 +420,14 @@ ncmpi_inq_varndims(int  ncid,    /* IN:  file ID */
     /* check whether variable ID is valid */
     if (varid < 0 || varid >= pncp->nvars) DEBUG_RETURN_ERROR(NC_ENOTVAR)
 
+    *ndimsp = pncp->vars[varid].ndims;
+    return NC_NOERR;
+
+#if 0
     /* calling the subroutine that implements ncmpi_inq_varndims() */
     return pncp->driver->inq_var(pncp->ncp, varid, NULL, NULL, ndimsp,
                                  NULL, NULL, NULL, NULL, NULL);
+#endif
 }
 
 /*----< ncmpi_inq_vardimid() >-----------------------------------------------*/
