@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h> /* strcpy(), strlen() */
+#include <errno.h>  /* EINVAL */
 #include <assert.h>
 
 #include <pnetcdf.h>
@@ -43,6 +44,13 @@ static void   *ncmpii_mem_root;
 static size_t  ncmpii_mem_alloc;
 static size_t  ncmpii_max_mem_alloc;
 
+#ifdef ENABLE_THREAD_SAFE
+/* updating the binary tree used in tfind()/tsearch()/tdelete() is not
+ * thread-safe, protect these subroutines with a mutex */
+#include<pthread.h>
+static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+#endif
+
 #if 0
 /*----< ncmpii_init_malloc_tracing() >----------------------------------------*/
 void ncmpii_init_malloc_tracing(void)
@@ -62,7 +70,7 @@ typedef struct {
     char   *filename;
 } ncmpii_mem_entry;
 
-inline static
+static
 int ncmpii_cmp(const void *a, const void *b) {
     ncmpii_mem_entry *fa = (ncmpii_mem_entry*)a;
     ncmpii_mem_entry *fb = (ncmpii_mem_entry*)b;
@@ -72,7 +80,7 @@ int ncmpii_cmp(const void *a, const void *b) {
     return 0;
 }
 
-inline static
+static
 void walker(const void *node, const VISIT which, const int depth) {
     ncmpii_mem_entry *f;
     f = *(ncmpii_mem_entry **)node;
@@ -89,6 +97,10 @@ void ncmpii_add_mem_entry(void       *buf,
                           const char *func,
                           const char *filename)
 {
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_lock(&lock);
+#endif
+
     /* use C tsearch utility */
     ncmpii_mem_entry *node = (ncmpii_mem_entry*) malloc(sizeof(ncmpii_mem_entry));
     node->self     = node;
@@ -107,10 +119,14 @@ void ncmpii_add_mem_entry(void       *buf,
     if (ret == NULL) {
         fprintf(stderr, "Error at line %d file %s: tsearch()\n",
                 __LINE__,__FILE__);
-        return;
     }
-    ncmpii_mem_alloc += size;
-    ncmpii_max_mem_alloc = MAX(ncmpii_max_mem_alloc, ncmpii_mem_alloc);
+    else {
+        ncmpii_mem_alloc += size;
+        ncmpii_max_mem_alloc = MAX(ncmpii_max_mem_alloc, ncmpii_mem_alloc);
+    }
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_unlock(&lock);
+#endif
 }
 
 /*----< ncmpii_del_mem_entry() >---------------------------------------------*/
@@ -118,6 +134,9 @@ void ncmpii_add_mem_entry(void       *buf,
 static
 void ncmpii_del_mem_entry(void *buf)
 {
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_lock(&lock);
+#endif
     /* use C tsearch utility */
     if (ncmpii_mem_root != NULL) {
         ncmpii_mem_entry node;
@@ -127,7 +146,7 @@ void ncmpii_del_mem_entry(void *buf)
         if (ret == NULL) {
             fprintf(stderr, "Error at line %d file %s: tfind() buf=%p\n",
                     __LINE__,__FILE__,buf);
-            return;
+            goto fn_exit;
         }
         /* free space for func and filename */
         free((*found)->func);
@@ -140,32 +159,37 @@ void ncmpii_del_mem_entry(void *buf)
         if (ret == NULL) {
             fprintf(stderr, "Error at line %d file %s: tdelete() buf=%p\n",
                     __LINE__,__FILE__,buf);
-            return;
+            goto fn_exit;
         }
         free(tmp);
     }
     else
         fprintf(stderr, "Error at line %d file %s: ncmpii_mem_root is NULL\n",
                 __LINE__,__FILE__);
+fn_exit:
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_unlock(&lock);
+#endif
+    return;
 }
 #endif
 
 /*----< NCI_Malloc_fn() >-----------------------------------------------------*/
+/* This subroutine is esstentially the same as calling malloc().
+ * According to malloc man page, If size is 0, then malloc() returns either
+ * NULL, or a unique pointer value that can later be successfully passed to
+ * free(). Thus, there is no need to check whether size is zero.
+ */
 void *NCI_Malloc_fn(size_t      size,
                     const int   lineno,
                     const char *func,
                     const char *filename)
 {
-#ifdef _DEBUG
-    if (size == 0)
-        fprintf(stderr, "Attempt to malloc zero-size in file %s func %s line %d\n", filename, func, lineno);
-#endif
-    if (size == 0) return NULL;
     void *buf = malloc(size);
-    if (!buf)
-	fprintf(stderr, "malloc(%zd) failed in file %s func %s line %d\n", size, filename, func, lineno);
 #ifdef PNETCDF_DEBUG
-    assert(buf != NULL);
+    if (size > 0 && buf == NULL)
+        fprintf(stderr, "malloc(%zd) failed in file %s func %s line %d\n",
+                size, filename, func, lineno);
 #endif
 #ifdef PNC_MALLOC_TRACE
     ncmpii_add_mem_entry(buf, size, lineno, func, filename);
@@ -175,22 +199,22 @@ void *NCI_Malloc_fn(size_t      size,
 
 
 /*----< NCI_Calloc_fn() >-----------------------------------------------------*/
+/* This subroutine is esstentially the same as calling calloc().
+ * According to calloc man page, If nelem is 0, then calloc() returns either
+ * NULL, or a unique pointer value that can later be successfully passed to
+ * free(). Thus, there is no need to check whether nelem is zero.
+ */
 void *NCI_Calloc_fn(size_t      nelem,
                     size_t      elsize,
                     const int   lineno,
                     const char *func,
                     const char *filename)
 {
-#ifdef _DEBUG
-    if (nelem == 0 || elsize == 0)
-        fprintf(stderr, "Attempt to calloc zero-size in file %s func %s line %d\n", filename, func, lineno);
-#endif
-    if (nelem == 0 || elsize == 0) return NULL;
-    void *buf =calloc(nelem, elsize);
-    if (!buf)
-	fprintf(stderr, "calloc(%zd, %zd) failed in file %s func %s line %d\n", nelem, elsize, filename, func, lineno);
+    void *buf = calloc(nelem, elsize);
 #ifdef PNETCDF_DEBUG
-    assert(buf != NULL);
+    if (nelem > 0 && buf == NULL)
+        fprintf(stderr, "calloc(%zd, %zd) failed in file %s func %s line %d\n",
+                nelem, elsize, filename, func, lineno);
 #endif
 #ifdef PNC_MALLOC_TRACE
     ncmpii_add_mem_entry(buf, nelem * elsize, lineno, func, filename);
@@ -200,25 +224,35 @@ void *NCI_Calloc_fn(size_t      nelem,
 
 
 /*----< NCI_Realloc_fn() >----------------------------------------------------*/
+/* This subroutine is esstentially the same as calling realloc().
+ * According to realloc man page, if ptr is NULL, then the call is equivalent
+ * to malloc(size), for all values of size; if size is equal to zero, and ptr
+ * is not NULL, then the call is equivalent to free(ptr). Unless ptr is NULL,
+ * it must have been returned by an earlier call to malloc(), calloc() or
+ * realloc(). If size was equal to 0, either NULL or a pointer suitable to be
+ * passed to free() is returned.
+ */
 void *NCI_Realloc_fn(void       *ptr,
                      size_t      size,
                      const int   lineno,
                      const char *func,
                      const char *filename)
 {
-#ifdef _DEBUG
-    if (size == 0)
-        fprintf(stderr, "Attempt to realloc zero-size in file %s func %s line %d\n", filename, func, lineno);
-#endif
+    if (ptr == NULL) return NCI_Malloc_fn(size, lineno, func, filename);
+
+    if (size == 0) {
+        NCI_Free_fn(ptr, lineno, func, filename);
+        return NCI_Malloc_fn(0, lineno, func, filename);
+    }
+
 #ifdef PNC_MALLOC_TRACE
-    if (ptr != NULL) ncmpii_del_mem_entry(ptr);
+    ncmpii_del_mem_entry(ptr);
 #endif
-    if (size == 0) return NULL;
     void *buf = (void *) realloc(ptr, size);
-    if (!buf)
-	fprintf(stderr, "realloc failed in file %s func %s line %d\n", filename, func, lineno);
 #ifdef PNETCDF_DEBUG
-    assert(buf != NULL);
+    if (buf == NULL)
+	fprintf(stderr, "realloc failed in file %s func %s line %d\n",
+                filename, func, lineno);
 #endif
 #ifdef PNC_MALLOC_TRACE
     ncmpii_add_mem_entry(buf, size, lineno, func, filename);
@@ -228,15 +262,17 @@ void *NCI_Realloc_fn(void       *ptr,
 
 
 /*----< NCI_Free_fn() >-------------------------------------------------------*/
+/* This subroutine is esstentially the same as calling free().
+ * According to free man page, free() frees the memory space pointed to by ptr,
+ * which must have been returned by a previous call to malloc(), calloc() or
+ * realloc(). Otherwise, or if free(ptr) has already been called before,
+ * undefined  behavior occurs. If ptr is NULL, no operation is performed.
+ */
 void NCI_Free_fn(void       *ptr,
                  const int   lineno,
                  const char *func,
                  const char *filename)
 {
-#ifdef _DEBUG
-    if (ptr == NULL)
-	fprintf(stderr, "Attempt to free null pointer in file %s func %s line %d\n", filename, func, lineno);
-#endif
     if (ptr == NULL) return;
 #ifdef PNC_MALLOC_TRACE
     ncmpii_del_mem_entry(ptr);
@@ -251,7 +287,13 @@ void NCI_Free_fn(void       *ptr,
 int ncmpi_inq_malloc_size(MPI_Offset *size)
 {
 #ifdef PNC_MALLOC_TRACE
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_lock(&lock);
+#endif
     *size = (MPI_Offset)ncmpii_mem_alloc;
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_unlock(&lock);
+#endif
     return NC_NOERR;
 #else
     DEBUG_RETURN_ERROR(NC_ENOTENABLED)
@@ -264,7 +306,13 @@ int ncmpi_inq_malloc_size(MPI_Offset *size)
 int ncmpi_inq_malloc_max_size(MPI_Offset *size)
 {
 #ifdef PNC_MALLOC_TRACE
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_lock(&lock);
+#endif
     *size = (MPI_Offset)ncmpii_max_mem_alloc;
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_unlock(&lock);
+#endif
     return NC_NOERR;
 #else
     DEBUG_RETURN_ERROR(NC_ENOTENABLED)
@@ -277,9 +325,15 @@ int ncmpi_inq_malloc_max_size(MPI_Offset *size)
 int ncmpi_inq_malloc_list(void)
 {
 #ifdef PNC_MALLOC_TRACE
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_lock(&lock);
+#endif
     /* check if malloc tree is empty */
     if (ncmpii_mem_root != NULL)
         twalk(ncmpii_mem_root, walker);
+#ifdef ENABLE_THREAD_SAFE
+    pthread_mutex_unlock(&lock);
+#endif
     return NC_NOERR;
 #else
     DEBUG_RETURN_ERROR(NC_ENOTENABLED)
