@@ -40,6 +40,12 @@
  * in an entire climate header in one go */
 #define NC_DEFAULT_CHUNKSIZE 262144
 
+/* default size of temporal buffer to pack noncontiguous user buffers for MPI
+ * collective read and write during ncmpi_wait/wait_all(). On some systems,
+ * e.g. Cray KNL, using contiguous user buffers in collective I/O is much
+ * faster than noncontiguous. */
+#define NC_DEFAULT_IBUF_SIZE 16777216
+
 /* when variable's nctype is NC_CHAR, I/O buffer's MPI type must be MPI_CHAR
  * and vice versa */
 #define NCMPII_ECHAR(nctype, mpitype) ((((nctype) == NC_CHAR) == ((mpitype) != MPI_CHAR)) ? NC_ECHAR : NC_NOERR)
@@ -303,7 +309,7 @@ typedef struct NC_req {
     MPI_Offset    nelems;       /* number of array elements requested */
     MPI_Offset   *start;        /* [varp->ndims*3] for start/count/stride */
     void         *xbuf;         /* buffer in external type, used in file I/O calls */
-    NC_lead_req  *lead;         /* point to lead request */
+    int           lead_off;     /* start index in the lead queue */
 } NC_req;
 
 #define NC_ABUF_DEFAULT_TABLE_SIZE 128
@@ -346,16 +352,22 @@ struct NC {
     struct NC    *ncp_sf;       /* ncp of subfile */
     MPI_Comm      comm_sf;      /* subfile MPI communicator */
 #endif
-    int           striping_unit; /* file stripe size of the file */
+    int           striping_unit; /* stripe size of the file */
     int           chunk;       /* chunk size for reading header */
-    MPI_Offset    h_align;     /* file alignment for header */
-    MPI_Offset    v_align;     /* file alignment for each fixed variable */
+    MPI_Offset    h_align;     /* file alignment for header size */
+    MPI_Offset    v_align;     /* alignment of the beginning of fixed-size variables */
+    MPI_Offset    fx_v_align;  /* file alignment for each fixed-size variable */
     MPI_Offset    r_align;     /* file alignment for record variable section */
     MPI_Offset    h_minfree;   /* pad at the end of the header section */
     MPI_Offset    v_minfree;   /* pad at the end of the data section for fixed-size variables */
-    MPI_Offset    xsz;       /* external size of this header, <= var[0].begin */
-    MPI_Offset    begin_var; /* file offset of the first (non-record) var */
-    MPI_Offset    begin_rec; /* file offset of the first 'record' */
+    MPI_Offset    ibuf_size;   /* packing buffer size for flushing noncontig
+                                  user buffer during wait */
+    MPI_Offset    xsz;         /* size of this file header, <= var[0].begin */
+    MPI_Offset    begin_var;   /* file offset of the first fixed-size variable,
+                                  if no fixed-sized variable, it is the offset
+                                  of first record variable. This value is also
+                                  the size of file header extent. */
+    MPI_Offset    begin_rec;   /* file offset of the first 'record' */
 
     MPI_Offset    recsize;   /* length of 'record': sum of single record sizes
                                 of all the record variables */
@@ -372,6 +384,8 @@ struct NC {
     NC_attrarray  attrs;    /* global attributes defined */
     NC_vararray   vars;     /* variables defined */
 
+    int           maxGetReqID;    /* max get request ID */
+    int           maxPutReqID;    /* max put request ID */
     int           numLeadGetReqs; /* number of pending lead get requests */
     int           numLeadPutReqs; /* number of pending lead put requests */
     NC_lead_req  *get_lead_list;  /* list of lead nonblocking read requests */
@@ -427,8 +441,9 @@ typedef struct bufferinfo {
     int         size;     /* allocated size of the buffer */
     int         version;  /* 1, 2, and 5 for CDF-1, 2, and 5 respectively */
     int         safe_mode;/* 0: disabled, 1: enabled */
-    void       *base;     /* beginning of read/write buffer */
-    void       *pos;      /* current position in buffer */
+    char       *base;     /* beginning of read/write buffer */
+    char       *pos;      /* current position in buffer */
+    char       *end;      /* end position of buffer */
 } bufferinfo;
 
 extern MPI_Offset
@@ -470,8 +485,8 @@ extern int
 ncmpio_abuf_dealloc(NC *ncp, int abuf_index);
 
 extern int
-ncmpio_add_record_requests(NC_req *reqs, MPI_Offset num_recs,
-                           const MPI_Offset *stride);
+ncmpio_add_record_requests(NC_lead_req *lead_list, NC_req *reqs,
+                           MPI_Offset num_recs, const MPI_Offset *stride);
 
 extern int
 ncmpio_igetput_varm(NC *ncp, NC_var *varp, const MPI_Offset *start,
@@ -544,7 +559,7 @@ ncmpio_close_files(NC *ncp, int doUnlink);
 
 /* Begin defined in ncmpio_utils.c ------------------------------------------*/
 extern void
-ncmpio_set_pnetcdf_hints(NC *ncp, MPI_Info info);
+ncmpio_set_pnetcdf_hints(NC *ncp, MPI_Info user_info, MPI_Info info_used);
 
 extern int
 ncmpio_NC_check_name(const char *name, int file_ver);
@@ -571,4 +586,10 @@ ncmpio_unpack_xbuf(int format, NC_var *varp, MPI_Offset bufcount,
                  MPI_Datatype etype, MPI_Datatype imaptype, int need_convert,
                  int need_swap, void *buf, void *xbuf);
 
+/* Begin defined in ncmpio_file_io.c ----------------------------------------*/
+extern int
+ncmpio_read_write(NC *ncp, int rw_flag, int coll_indep, MPI_Offset offset,
+                  int len, MPI_Datatype buf_type, void *buf,
+                  int buftype_is_contig);
+
 #endif /* _NC_H */
