@@ -79,7 +79,11 @@ move_file_block(NC         *ncp,
             nbytes -= chunk_size*nprocs;
         }
 
-        /* explicitly initialize mpistatus object to 0, see comments below */
+        /* explicitly initialize mpistatus object to 0. For zero-length read,
+         * MPI_Get_count may report incorrect result for some MPICH version,
+         * due to the uninitialized MPI_Status object passed to MPI-IO calls.
+         * Thus we initialize it above to work around.
+         */
         memset(&mpistatus, 0, sizeof(MPI_Status));
 
         /* read the original data @ from+nbytes+rank*chunk_size */
@@ -130,7 +134,11 @@ move_file_block(NC         *ncp,
          * that have not been written before. Below uses the former option.
          */
 #ifdef _USE_MPI_GET_COUNT
-        /* explicitly initialize mpistatus object to 0, see comments below */
+        /* explicitly initialize mpistatus object to 0. For zero-length read,
+         * MPI_Get_count may report incorrect result for some MPICH version,
+         * due to the uninitialized MPI_Status object passed to MPI-IO calls.
+         * Thus we initialize it above to work around.
+         */
         memset(&mpistatus, 0, sizeof(MPI_Status));
 #endif
         TRACE_IO(MPI_File_write_at_all)(ncp->collective_fh,
@@ -142,11 +150,6 @@ move_file_block(NC         *ncp,
             if (err == NC_EFILE) DEBUG_ASSIGN_ERROR(status, NC_EWRITE)
         }
         else {
-            /* for zero-length read, MPI_Get_count may report incorrect result
-             * for some MPICH version, due to the uninitialized MPI_Status
-             * object passed to MPI-IO calls. Thus we initialize it above to
-             * work around.
-             */
 #ifdef _USE_MPI_GET_COUNT
             int put_size;
             MPI_Get_count(&mpistatus, MPI_BYTE, &put_size);
@@ -277,7 +280,11 @@ NC_begins(NC *ncp)
      * Note ncp->xsz is header size and ncp->begin_var is header extent.
      * Add the minimum header free space requested by user.
      */
-    ncp->begin_var = D_RNDUP(ncp->xsz + ncp->h_minfree, ncp->h_align);
+    if (ncp->vars.ndefined > 0)
+        ncp->begin_var = D_RNDUP(ncp->xsz + ncp->h_minfree, ncp->h_align);
+    else /* no variable defined, ignore alignment and set header extent to
+          * header size */
+        ncp->begin_var = ncp->xsz;
 
     if (ncp->old != NULL) {
         /* If this define mode was entered from a redef(), we check whether
@@ -307,7 +314,7 @@ NC_begins(NC *ncp)
         /* this will pad out non-record variables with zero to the
          * requested alignment.  record variables are a bit trickier.
          * we don't do anything special with them */
-        ncp->vars.value[i]->begin = D_RNDUP(end_var, ncp->v_align);
+        ncp->vars.value[i]->begin = D_RNDUP(end_var, ncp->fx_v_align);
 
         if (ncp->old != NULL) {
             /* move to the next fixed variable */
@@ -336,12 +343,12 @@ NC_begins(NC *ncp)
      * begin_rec as is (in case some non-record variables are deleted)
      */
     if (ncp->begin_rec < end_var ||
-        ncp->begin_rec != D_RNDUP(ncp->begin_rec, ncp->v_align))
-        ncp->begin_rec = D_RNDUP(end_var, ncp->v_align);
+        ncp->begin_rec != D_RNDUP(ncp->begin_rec, ncp->fx_v_align))
+        ncp->begin_rec = D_RNDUP(end_var, ncp->fx_v_align);
 
     /* expand free space for fixed variable section */
     if (ncp->begin_rec < end_var + ncp->v_minfree)
-        ncp->begin_rec = D_RNDUP(end_var + ncp->v_minfree, ncp->v_align);
+        ncp->begin_rec = D_RNDUP(end_var + ncp->v_minfree, ncp->fx_v_align);
 
     /* align the starting offset for record variable section */
     if (ncp->r_align > 1)
@@ -445,128 +452,68 @@ NC_begins(NC *ncp)
  * This function is collective and only called by enddef().
  * Write out the header
  * 1. Call ncmpio_hdr_put_NC() to copy the header object, ncp, to a buffer.
- * 2. Call NC_check_header() to check if header is consistent across all
- *    processes.
- * 3. Process rank 0 writes the header to file.
- * This is a collective call.
+ * 2. Process rank 0 writes the header to file.
  */
 static int
 write_NC(NC *ncp)
 {
-    void *buf;
-    int status, mpireturn, err, rank;
-    MPI_Offset local_xsz;
+    void *buf=NULL;
+    int status=NC_NOERR, mpireturn, err, rank, header_wlen;
+    size_t bufLen;
+    MPI_Status mpistatus;
 
     assert(!NC_readonly(ncp));
 
-    /* In NC_begins(), root's ncp->xsz, root's header size, has been
-     * broadcast, so ncp->xsz is now root's header size. To check any
-     * inconsistency in file header, we need to calculate local header
-     * size by calling ncmpio_hdr_len_NC()./
-     */
-    local_xsz = ncmpio_hdr_len_NC(ncp);
-
-    /* Note valgrind will complain about uninitialized buf below, but buf will
-     * be first filled with header of size ncp->xsz and later write to file.
-     * So, no need to change to NCI_Calloc for the sake of valgrind.
-     */
-    buf = NCI_Malloc((size_t)local_xsz); /* buffer for local header object */
-
-    /* copy the entire local header object to buf */
-    status = ncmpio_hdr_put_NC(ncp, buf);
-    if (status != NC_NOERR) { /* a fatal error */
-        NCI_Free(buf);
-        return status;
-    }
+    if (ncp->begin_var > X_INT_MAX) /* a fatal error */
+        DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
 
     MPI_Comm_rank(ncp->comm, &rank);
 
-#ifdef _DIFF_HEADER
-    if (ncp->safe_mode == 0) {
-        int h_size=(int)ncp->xsz;
-        void *root_header;
-
-        /* check header against root's */
-        if (rank == 0) root_header = buf;
-        else           root_header = (void*) NCI_Malloc((size_t)h_size);
-
-        TRACE_COMM(MPI_Bcast)(root_header, h_size, MPI_BYTE, 0, ncp->comm);
-        if (mpireturn != MPI_SUCCESS) {
-            err = ncmpii_error_mpi2nc(mpireturn, "MPI_Bcast");
-            DEBUG_RETURN_ERROR(err)
-        }
-
-        if (rank > 0) {
-            if (h_size != local_xsz || memcmp(buf, root_buf, h_size))
-                status = NC_EMULTIDEFINE;
-            NCI_Free(root_buf);
-        }
-
-        /* report error if header is inconsistency */
-        TRACE_COMM(MPI_Allreduce)(&status, &err, 1, MPI_INT, MPI_MIN,ncp->comm);
-        if (mpireturn != MPI_SUCCESS) {
-            err = ncmpii_error_mpi2nc(mpireturn, "MPI_Allreduce");
-            DEBUG_RETURN_ERROR(err)
-        }
-        if (err != NC_NOERR) {
-            /* TODO: this error return is harsh. Maybe relax for inconsistent
-             * attribute contents? */
-            if (status == NC_NOERR) status = err;
-            NCI_Free(buf);
-            return status;
-        }
-    }
-#endif
-
-#ifdef _CHECK_HEADER_CONSISTENCY
-    /* check the header consistency across all processes and sync header.
-     * When safe_mode is on:
-     *   The returned status on root can be either NC_NOERR (all headers are
-     *   consistent) or NC_EMULTIDEFINE (some headers are inconsistent).
-     *   The returned status on non-root processes can be NC_NOERR, fatal
-     *   error (>-250), or inconsistency error (-250 to -269).
-     * When safe_mode is off:
-     *   The returned status on root is always NC_NOERR
-     *   The returned status on non-root processes can be NC_NOERR, fatal
-     *   error (>-250), or inconsistency error (-250 to -269).
-     * For fatal error, we should stop. For others, we can continue.
-     */
-    int max_err;
-    status = NC_check_header(ncp, buf, local_xsz);
-
-    /* check for fatal error */
-    err =  (status != NC_NOERR && !ErrIsHeaderDiff(status)) ? 1 : 0;
-    max_err = err;
-
-    if (ncp->safe_mode == 1) {
-        TRACE_COMM(MPI_Allreduce)(&err, &max_err, 1, MPI_INT, MPI_MAX,
-                                  ncp->comm);
-        if (mpireturn != MPI_SUCCESS) {
-            err = ncmpii_error_mpi2nc(mpireturn,"MPI_Allreduce");
-            DEBUG_RETURN_ERROR(err)
-        }
-    }
-    if (max_err == 1) { /* some processes encounter a fatal error */
-        NCI_Free(buf);
-        return status;
-    }
-#endif
-    /* For non-fatal error, we continue to write header to the file, as now the
-     * header object in memory has been sync-ed across all processes. */
-
     /* only rank 0's header gets written to the file */
     if (rank == 0) {
+
+        /* In NC_begins(), root's ncp->xsz and ncp->begin_var, root's header
+         * size and extent, have been broadcast (sync-ed) among processes.
+         */
+#ifdef ENABLE_NULL_BYTE_HEADER_PADDING
+        /* NetCDF classic file formats require the file header null-byte
+         * padded. Thus we must calloc a buffer of size equal to file header
+         * extent.
+         */
+        header_wlen = (int) ncp->begin_var;
+        bufLen = _RNDUP(header_wlen, X_ALIGN);
+        buf = NCI_Calloc(bufLen, 1);
+#else
+        /* Do not write padding area (between ncp->xsz and ncp->begin_var) */
+        header_wlen = (int) ncp->xsz;
+        bufLen = _RNDUP(header_wlen, X_ALIGN);
+        buf = NCI_Malloc(bufLen);
+#endif
+
+        /* copy the entire local header object to buf */
+        status = ncmpio_hdr_put_NC(ncp, buf);
+        if (status != NC_NOERR) /* a fatal error */
+            goto fn_exit;
+
+        /* For non-fatal error, we continue to write header to the file, as now
+         * the header object in memory has been sync-ed across all processes.
+         */
+
         /* rank 0's fileview already includes the file header */
-        MPI_Status mpistatus;
-        if (ncp->xsz != (int)ncp->xsz)
-            DEBUG_ASSIGN_ERROR(status, NC_EINTOVERFLOW)
 
 #ifdef _USE_MPI_GET_COUNT
-        /* explicitly initialize mpistatus object to 0, see comments below */
+        /* explicitly initialize mpistatus object to 0. For zero-length read,
+         * MPI_Get_count may report incorrect result for some MPICH version,
+         * due to the uninitialized MPI_Status object passed to MPI-IO calls.
+         * Thus we initialize it above to work around.
+         */
         memset(&mpistatus, 0, sizeof(MPI_Status));
 #endif
+        /* write the header extent, not just header size, because NetCDF file
+         * format specification requires null byte padding for header.
+         */
         TRACE_IO(MPI_File_write_at)(ncp->collective_fh, 0, buf,
-                                    (int)ncp->xsz, MPI_BYTE, &mpistatus);
+                                    header_wlen, MPI_BYTE, &mpistatus);
         if (mpireturn != MPI_SUCCESS) {
             err = ncmpii_error_mpi2nc(mpireturn, "MPI_File_write_at");
             /* write has failed, which is more serious than inconsistency */
@@ -578,11 +525,14 @@ write_NC(NC *ncp)
             MPI_Get_count(&mpistatus, MPI_BYTE, &put_size);
             ncp->put_size += put_size;
 #else
-            ncp->put_size += ncp->xsz;
+            ncp->put_size += header_wlen;
 #endif
         }
     }
 
+fn_exit:
+    if (buf != NULL) NCI_Free(buf);
+
     if (ncp->safe_mode == 1) {
         /* broadcast root's status, because only root writes to the file */
         int root_status = status;
@@ -592,7 +542,6 @@ write_NC(NC *ncp)
     }
 
     fClr(ncp->flags, NC_NDIRTY);
-    NCI_Free(buf);
 
     return status;
 }
@@ -804,7 +753,7 @@ ncmpio_NC_check_voffs(NC *ncp)
 
     if (ncp->begin_rec < prev_off) {
         if (ncp->safe_mode)
-            printf("Record variable section begin offset (%lld) is less than fix-sized variable section end offset (%lld)\n", varp->begin, prev_off);
+            printf("Record variable section begin offset (%lld) is less than fixed-size variable section end offset (%lld)\n", varp->begin, prev_off);
         DEBUG_RETURN_ERROR(NC_ENOTNC)
     }
 
@@ -836,7 +785,15 @@ ncmpio_NC_check_voffs(NC *ncp)
 }
 
 /*----< ncmpio__enddef() >---------------------------------------------------*/
-/* This is a collective subroutine. */
+/* This is a collective subroutine.
+ * h_minfree  Sets the pad at the end of the "header" section.
+ * v_align    Controls the alignment of the beginning of the data section for
+ *            fixed size variables.
+ * v_minfree  Sets the pad at the end of the data section for fixed size
+ *            variables.
+ * r_align    Controls the alignment of the beginning of the data section for
+ *            variables which have an unlimited dimension (record variables).
+ */
 int
 ncmpio__enddef(void       *ncdp,
                MPI_Offset  h_minfree,
@@ -870,7 +827,7 @@ ncmpio__enddef(void       *ncdp,
     }
     ncp->striping_unit = striping_unit;
 
-    all_fix_var_size = 0;  /* sum of all defined fix-sized variables */
+    all_fix_var_size = 0;  /* sum of all defined fixed-size variables */
     for (i=0; i<ncp->vars.ndefined; i++) {
         if (IS_RECVAR(ncp->vars.value[i])) continue;
         all_fix_var_size += ncp->vars.value[i]->len;
@@ -880,7 +837,11 @@ ncmpio__enddef(void       *ncdp,
      * set during file create/open */
 
     if (ncp->h_align == 0) { /* user info does not set nc_header_align_size */
-        if (striping_unit &&
+        if (v_align > 0)
+            ncp->h_align = v_align;
+        else if (all_fix_var_size > 0 && r_align > 0) /* no fixed-size variable */
+            ncp->h_align = r_align;
+        else if (striping_unit &&
             all_fix_var_size > FILE_ALIGNMENT_LB * striping_unit)
             /* if striping_unit is available and file size sufficiently large */
             ncp->h_align = striping_unit;
@@ -892,37 +853,46 @@ ncmpio__enddef(void       *ncdp,
     if (ncp->v_align == 0) { /* user info does not set nc_var_align_size */
         if (v_align > 0) /* else respect user hint */
             ncp->v_align = v_align;
+#ifdef USE_AGGRESSIVE_ALIGN
         else if (striping_unit &&
                  all_fix_var_size > FILE_ALIGNMENT_LB * striping_unit)
             /* if striping_unit is available and file size sufficiently large */
             ncp->v_align = striping_unit;
         else
             ncp->v_align = FILE_ALIGNMENT_DEFAULT;
+#endif
     }
 
-    if (ncp->r_align == 0) { /* user info does not set nc_record_align_size */
+    if (ncp->r_align == 0) { /* user info/env does not set nc_record_align_size */
         if (r_align > 0) /* else respect user hint */
             ncp->r_align = r_align;
+#ifdef USE_AGGRESSIVE_ALIGN
         if (striping_unit)
             ncp->r_align = striping_unit;
         else
             ncp->r_align = FILE_ALIGNMENT_DEFAULT;
+#endif
     }
+    /* else ncp->r_align is already set by user/env, ignore the one passed by
+     * the argument r_align of this subroutine.
+     */
 
     /* all CDF formats require 4-bytes alignment */
-    if (ncp->h_align == 0) ncp->h_align = 4;
-    else                   ncp->h_align = D_RNDUP(ncp->h_align, 4);
-    if (ncp->v_align == 0) ncp->v_align = 4;
-    else                   ncp->v_align = D_RNDUP(ncp->v_align, 4);
-    if (ncp->r_align == 0) ncp->r_align = 4;
-    else                   ncp->r_align = D_RNDUP(ncp->r_align, 4);
+    if (ncp->h_align == 0)    ncp->h_align = 4;
+    else                      ncp->h_align = D_RNDUP(ncp->h_align, 4);
+    if (ncp->v_align == 0)    ncp->v_align = 4;
+    else                      ncp->v_align = D_RNDUP(ncp->v_align, 4);
+    if (ncp->fx_v_align == 0) ncp->fx_v_align = 4;
+    else                      ncp->fx_v_align = D_RNDUP(ncp->fx_v_align, 4);
+    if (ncp->r_align == 0)    ncp->r_align = 4;
+    else                      ncp->r_align = D_RNDUP(ncp->r_align, 4);
 
     /* reflect the hint changes to the MPI info object, so the user can inquire
      * what the true hint values are being used
      */
     sprintf(value, "%lld", ncp->h_align);
     MPI_Info_set(ncp->mpiinfo, "nc_header_align_size", value);
-    sprintf(value, "%lld", ncp->v_align);
+    sprintf(value, "%lld", ncp->fx_v_align);
     MPI_Info_set(ncp->mpiinfo, "nc_var_align_size", value);
     sprintf(value, "%lld", ncp->r_align);
     MPI_Info_set(ncp->mpiinfo, "nc_record_align_size", value);
@@ -935,6 +905,9 @@ ncmpio__enddef(void       *ncdp,
         err = ncmpio_subfile_partition(ncp);
         CHECK_ERROR(err)
     }
+#else
+    MPI_Info_set(ncp->mpiinfo, "pnetcdf_subfiling", "disable");
+    MPI_Info_set(ncp->mpiinfo, "nc_num_subfiles", "0");
 #endif
 
     /* check whether sizes of all variables are legal */
