@@ -32,17 +32,7 @@
     } \
 }
 
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
-
-typedef struct {
-	char infname[PATH_MAX];
-	char outfname[PATH_MAX];
-} params;
-
-void parse_read_args(int argc, char **argv, int rank, params *p);
-void parse_write_args(int argc, char **argv, int rank, params *p);
+int inq_env_hint(char *hint_key, char **hint_value);
 
 #ifdef PNETCDF_DEBUG
 #define PASS_STR "\x1b[32mpass\x1b[0m\n"
@@ -68,4 +58,11 @@ extern char* nc_err_code_name(int err);
 #define MPI_OFFSET MPI_LONG_LONG_INT
 #endif
 
+#ifndef HAVE_STRDUP
+extern char *strdup(const char *s);
+#endif
+#ifndef HAVE_STRCASECMP
+extern int strcasecmp(const char *s1, const char *s2);
+#endif
+
 #endif
