@@ -408,6 +408,9 @@ int main(int argc, char **argv)
     rank_dim[2] =  rank %  array_of_psizes[2];
     rank_dim[1] = (rank /  array_of_psizes[2]) % array_of_psizes[1];
     rank_dim[0] =  rank / (array_of_psizes[2]  * array_of_psizes[1]);
+    if (verbose)
+        printf("rank %d: rank_dim[3]=%d %d %d\n",
+               rank,rank_dim[0],rank_dim[2],rank_dim[2]);
 
     /* starting coordinates of the subarray in each dimension */
     for (i=0; i<ndims; i++)
@@ -421,6 +424,9 @@ int main(int argc, char **argv)
         bufcounts[i] = bufcount;
         datatype_list[i] = MPI_INT;
     }
+    if (verbose)
+        printf("rank %d: starts[0][3]=%lld %lld %lld counts[0][3]=%lld %lld %lld\n",
+               rank,starts[0][0],starts[0][2],starts[0][2], counts[0][0],counts[0][1],counts[0][2]);
 
     buf[0] = (int *) malloc(bufcount * nvars * sizeof(int));
     if (buf[0] == NULL) {
@@ -452,7 +458,7 @@ int main(int argc, char **argv)
     MPI_Info_set(info, "romio_no_indep_rw", "true");
     MPI_Info_set(info, "romio_cb_write", "true");
  */
-    for (k=0; k<=9; k++){
+    for (k=0; k<=9; k++) {
         sprintf(filename, "%s.%d.%d.%d.nc", fbasename, length, nvars, k);
         if (k==0)
             strcpy(filename1, filename);
@@ -481,30 +487,41 @@ int main(int argc, char **argv)
 
         /* define variables */
         if (k<7){
-            for (i=0; i<2; i++){
+            for (i=0; i<2; i++){ /* define fixed-size variables */
                 sprintf(varname, "var0_%d", i);
                 err = ncmpi_def_var(ncid, varname, NC_INT, ndims, dimids0, &varid[i]);
                 CHECK_ERR
             }
-            for (i=2; i<nvars; i++){
+            for (i=2; i<nvars; i++){  /* define record variables */
                 sprintf(varname, "var1_%d", i);
                 err = ncmpi_def_var(ncid, varname, NC_INT, ndims, dimids1, &varid[i]);
                 CHECK_ERR
             }
         } else {
-            for (i=0; i<nprocs; i++){
+            for (i=0; i<nprocs; i++){ /* define fixed-size variables */
                 sprintf(varname, "var0_%d", i);
                 err = ncmpi_def_var(ncid, varname, NC_INT, ndims, dimids0, &varid[i]);
                 CHECK_ERR
             }
         }
 
+        if (k == 0) {
+            err = ncmpi_set_fill(ncid, NC_FILL, NULL); CHECK_ERR
+        }
+
         err = ncmpi_enddef(ncid);
         CHECK_ERR
 
         if (k == 0) {
             if (rank == 0 && verbose)
                 printf("*** Testing to write 2 non-record variables and 2 record variables by using ncmpi_put_vara_all() ...");
+            for (i=2; i<nvars; i++){
+                /* fill record variables to silence valgrind complaining about uninitialised bytes */
+                for (j=0; j<array_of_gsizes[0]; j++) {
+                    err = ncmpi_fill_var_rec(ncid, varid[i], j);
+                    CHECK_ERR
+                }
+            }
             for (i=0; i<nvars; i++){
                 err = ncmpi_put_vara_all(ncid, varid[i], starts[i], counts[i], buf[i], bufcounts[i], MPI_INT);
                 CHECK_ERR
@@ -623,12 +640,12 @@ int main(int argc, char **argv)
         err = ncmpi_close(ncid);
         CHECK_ERR
 
-        if (err == NC_NOERR){
-            if ((k>0)&&(k<7)){
-            err = ncmpi_diff(filename1, filename3);
-            if (rank == 0 && err == NC_NOERR && verbose)
-                printf("\t OK\n");
-            } else if (k>7){
+        if (err == NC_NOERR) {
+            if (k > 0 && k < 7) {
+                err = ncmpi_diff(filename1, filename3);
+                if (rank == 0 && err == NC_NOERR && verbose)
+                    printf("\t OK\n");
+            } else if (k > 7) {
 /*
 printf("filename2=%s filename3=%s\n",filename2, filename3);
                 err = ncmpi_diff(filename2, filename3);
@@ -687,7 +704,6 @@ printf("filename2=%s filename3=%s\n",filename2, filename3);
         }
     }
 
-ncmpi_inq_malloc_list();
     MPI_Allreduce(MPI_IN_PLACE, &nerrs, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
     if (rank == 0) {
         if (nerrs) printf(FAIL_STR,nerrs);
