@@ -13,18 +13,39 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h> /* strcasecmp() */
 #include <libgen.h> /* basename() */
 #include <mpi.h>
 #include <pnetcdf.h>
 
 #include <testutils.h>
 
+static int
+tst_fmt(char *filename, int flag)
+{
+    int err, nerrs=0, ncid, cmode;
+
+    MPI_Barrier(MPI_COMM_WORLD);
+
+    /* create a file if it does not exist */
+    cmode = NC_CLOBBER | flag;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
+    CHECK_ERR
+    err = ncmpi_close(ncid); CHECK_ERR
+
+    /* now the file exists, test if PnetCDF can return correct error code */
+    cmode = NC_NOCLOBBER | flag;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
+    EXP_ERR(NC_EEXIST) /* err == NC_EOFILE */
+
+    return nerrs;
+}
+
 int main(int argc, char **argv) {
-    char filename[256];
-    int  err, nerrs=0, ncid, cmode, rank, nprocs;
+    char filename[256], *hint_value;
+    int  err, nerrs=0, rank, bb_enabled=0;
 
     MPI_Init(&argc, &argv);
-    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
 
     if (argc > 2) {
@@ -43,16 +64,21 @@ int main(int argc, char **argv) {
         free(cmd_str);
     }
 
-    /* create a file if it does not exist */
-    cmode = NC_CLOBBER;
-    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
-    CHECK_ERR
-    err = ncmpi_close(ncid); CHECK_ERR
+    /* check whether burst buffering is enabled */
+    if (inq_env_hint("nc_burst_buf", &hint_value)) {
+        if (strcasecmp(hint_value, "enable") == 0) bb_enabled = 1;
+        free(hint_value);
+    }
 
-    /* now the file exists, test if PnetCDF can return correct error code */
-    cmode = NC_NOCLOBBER;
-    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
-    EXP_ERR(NC_EEXIST) /* err == NC_EOFILE */
+    nerrs += tst_fmt(filename, 0);
+    nerrs += tst_fmt(filename, NC_64BIT_OFFSET);
+    if (!bb_enabled) {
+#ifdef ENABLE_NETCDF4
+        nerrs += tst_fmt(filename, NC_NETCDF4);
+        nerrs += tst_fmt(filename, NC_NETCDF4|NC_CLASSIC_MODEL);
+#endif
+    }
+    nerrs += tst_fmt(filename, NC_64BIT_DATA);
 
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
